 /*
 * Copyright (C) 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

/** @cond OLPLUGIN */
/** @addtogroup UI */
/** @{ */

#include "afxwin.h"
#include "afxcmn.h"
#include "AnimatedIcon.h"

#if !defined(AFX_FORM1_H__FA98B71B_D0B7_11D3_BC39_00C04F602FEE__INCLUDED_)
#define AFX_FORM1_H__FA98B71B_D0B7_11D3_BC39_00C04F602FEE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/////////////////////////////////////////////////////////////////////////////
// CSyncForm form view

#ifndef __AFXEXT_H__
#include <afxext.h>
#endif

#include "CustomPane.h"


/**
 * Form of the main window.
 * Contains objects on the main screen of the UI (panes, labels, ...).
 */
class CSyncForm : public CFormView
{
protected:
	CSyncForm();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CSyncForm)

public:
	//{{AFX_DATA(CSyncForm)
	enum { IDD = IDD_SYNC_FORM };
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA

public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSyncForm)
protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual void OnDraw(CDC* pDC);

	//}}AFX_VIRTUAL

    virtual ~CSyncForm();

#ifdef _DEBUG
     virtual void AssertValid() const;
     virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
	//{{AFX_MSG(CSyncForm)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG
   DECLARE_MESSAGE_MAP()

private:
    // true if the UI buttons are locked
    bool lockedUI;

    // Buffers for source panes status labels
    CString contactsStatusLabel;
    CString calendarStatusLabel;
    CString tasksStatusLabel;
    CString notesStatusLabel;

    // Buffers for source panel titles (fixed)
    CString contactsLabel;
    CString calendarLabel;
    CString tasksLabel;
    CString notesLabel;

public:
    CFont fontBold;
    CFont fontNormal;
    CAnimatedIcon butStart;
    CAnimatedIcon iconContacts;
    CAnimatedIcon iconCalendar;
    CAnimatedIcon iconTasks;
    CAnimatedIcon iconNotes;
    CBrush  brushHollow;

    // right hand status icons
    CAnimatedIcon iconStatusContacts;
    CAnimatedIcon iconStatusCalendar;
    CAnimatedIcon iconStatusTasks;
    CAnimatedIcon iconStatusNotes;
    CAnimatedIcon iconStatusSync;

    // panes
    CCustomPane paneSync;
    CCustomPane paneContacts;
    CCustomPane paneCalendar;
    CCustomPane paneTasks;
    CCustomPane paneNotes;

    // sync source states {SYNCSOURCE_STATE_OK, SYNCSOURCE_STATE_NOT_SYNCED, SYNCSOURCE_STATE_CANCELED}
    // defined in winmaincpp.h        
    int syncSourceContactState, syncSourceCalendarState;
    int syncSourceTaskState, syncSourceNoteState;

    
    /**
    * refresh UI info about a source
    * @param sourceId : the source type id {SYNCSOURCE_CONTACTS,..}, defined in ClientUtil.h
    */
    void refreshSource(int sourceId);

    // refresh UI info about all sources, calls refreshSource for every sync source
    void refreshSources();
    
    /**
    * repaints sync controls from a pane associated to a source
    * @param paneType : the pane type id {PANE_TYPE_CONTACTS, ..}, defined in CustomPane.h
    */
    void repaintPaneControls(int paneType);

    // shows/hides the sync controls
    void showSyncControls(BOOL show);

    void lockButtons();
    void unlockButtons();

    // change UI status text for a source
    void changeContactsStatus(CString& status);
    void changeCalendarStatus(CString& status);
    void changeTasksStatus   (CString& status);
    void changeNotesStatus   (CString& status);

    afx_msg LRESULT OnInitForm(WPARAM, LPARAM);
    afx_msg void OnBnClickedMainButSync();
    afx_msg void OnNcPaint( );
    afx_msg HBRUSH OnCtlColor( CDC* pDC, CWnd* pWnd, UINT nCtlColor);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);

    // when main sync pane is clicked
    afx_msg void OnStnClickedMainBkSync();

    // when sync source panes are clicked
    afx_msg void OnStnClickedMainBkContacts();
    afx_msg void OnStnClickedMainBkCalendar();
    afx_msg void OnStnClickedMainBkTasks();
    afx_msg void OnStnClickedMainBkNotes();

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

/** @} */
/** @endcond */
#endif // !defined(AFX_FORM1_H__FA98B71B_D0B7_11D3_BC39_00C04F602FEE__INCLUDED_)
