 /*
 * Copyright (C) 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#pragma once
#include "afxwin.h"

/** @cond OLPLUGIN */
/** @addtogroup UI */
/** @{ */


/**
 * Returns the scheduler # of minutes, given the droplist position.
 * If minutes is different from the exact values, we go to the upper value available.
 * Default value is 15 minutes.
 */
int getSchedulerMinutes(int position);

/**
 * Returns the scheduler droplist position, given the # of minutes.
 * Position is 0 to 11 (default = 2);
 */
int getSchedulerPosition(int minutes);



/**
 * Sync options panel.
 */
class CSyncSettings : public CFormView
{
	DECLARE_DYNCREATE(CSyncSettings)

protected:
	CSyncSettings();           // protected constructor used by dynamic creation
	virtual ~CSyncSettings();

public:
	enum { IDD = IDD_SYNC };
#ifdef _DEBUG
	virtual void AssertValid() const;
#ifndef _WIN32_WCE
	virtual void Dump(CDumpContext& dc) const;
    virtual void PostNcDestroy( ){delete this;}
#endif
#endif

protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    virtual BOOL PreTranslateMessage(MSG* pMsg);
    afx_msg LRESULT OnInitForm(WPARAM, LPARAM);

	DECLARE_MESSAGE_MAP()
public:
    
    CButton checkContacts;
    CButton checkCalendar;
    CButton checkTasks;
    CButton checkNotes;
    CButton butContacts;
    CButton butCalendar;
    CButton butTasks;
    CButton butNotes;
    bool saveSyncTypeContacts;
    bool saveSyncTypeCalendar;
    bool saveSyncTypeTasks;
    bool saveSyncTypeNotes;
    bool saveScheduler; // true if scheduler settings have changed

    // scheduler
    CButton checkEnabled;
    CButton checkEncryption;
    CStatic groupItems;
    CStatic groupScheduler;
    CStatic groupSecurity;
    CComboBox comboSchedulerValue;
    
    bool saveSettings(bool);

    afx_msg void OnBnClickedSyncCheckContacts();
    afx_msg void OnBnClickedSyncCheckCalendar();
    afx_msg void OnBnClickedSyncCheckTasks();
    afx_msg void OnBnClickedSyncCheckNotes();
    afx_msg void OnBnClickedSyncOk();
    afx_msg void OnBnClickedSyncCancel();
    afx_msg void OnBnClickedSyncButContacts();
    afx_msg void OnBnClickedSyncButCalendar();
    afx_msg void OnBnClickedSyncButTasks();
    afx_msg void OnBnClickedSyncButNotes();
    afx_msg void OnNcPaint();   
  
    afx_msg void OnBnClickedSchedulerCheckEnabled();
    afx_msg void OnCbnSelchangeSchedulerComboValue();
};


/** @} */
/** @endcond */
