 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#include "base/Log.h"
#include "base/util/utils.h"
#include "SyncException.h"
#include <string>

using namespace std;

// Init static pointer.
SyncException* SyncException::pinstance = NULL;


/**
 * Throws a sync exception.
 * Get the instance of SyncException, set the data and throw it.
 */
void throwSyncException(const char* msg, DWORD code) {

    SyncException* e = SyncException::getInstance();
    e->setExceptionData(msg, code);
    throw e;
}



/// Method to create the sole instance of SyncException
SyncException* SyncException::getInstance() {
    if (pinstance == NULL) {
        pinstance = new SyncException;
    }
    return pinstance;
}


/// Constructor
SyncException::SyncException() {
    errorMsg  = NULL;
    werrorMsg = NULL;
    clear();
}


/// Destructor
SyncException::~SyncException() {

    if (errorMsg) {
        delete []  errorMsg;  errorMsg = NULL;
    }
    if (werrorMsg) {
        delete [] werrorMsg; werrorMsg = NULL;
    }
}



/// Set members of exception
void SyncException::setExceptionData(const char* msg, DWORD code) {

    if (msg) {
        if (errorMsg) {
            delete []  errorMsg;  errorMsg = NULL;
        }
        if (werrorMsg) {
            delete [] werrorMsg; werrorMsg = NULL;
        }
        errorMsg  = stringdup(msg);
        werrorMsg = toWideChar(msg);
    }
    errorCode = code;
}


/// Clear data of exception
void SyncException::clear() {

    if (errorMsg) {
        delete []  errorMsg;  errorMsg = NULL;
    }
    if (werrorMsg) {
        delete [] werrorMsg; werrorMsg = NULL;
    }
    errorMsg      = stringdup("");
    werrorMsg     = wstrdup (L"");
    errorCode     = NULL;
}




//
// Methods to get exception informations
//
const char* SyncException::getErrorMsg() {
    return errorMsg;
}

const WCHAR* SyncException::wgetErrorMsg() {
    return werrorMsg;
}

const wstring SyncException::wstrgetErrorMsg() {
    wstring ws = werrorMsg;
    return ws;
}



DWORD SyncException::getErrorCode() {
    return errorCode;
}

