 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#include "base/Log.h"
#include "event/OutlookSyncSourceListener.h"
#include "utils.h"
#include "HwndFunctions.h"


void OutlookSyncSourceListener::syncSourceBegin(SyncSourceEvent &event) {

    char* syncMode = syncModeName((SyncMode)event.getSyncMode());
	LOG.debug("---------------------- %s BEGIN (mode \"%s\") ----------------------", event.getSourceName(), syncMode);

    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SYNCSOURCE_BEGIN, 
                NULL, (LPARAM) syncSourceNameToIndex(event.getSourceName()));
}

void OutlookSyncSourceListener::syncSourceEnd(SyncSourceEvent &event) {

    char* syncMode = syncModeName((SyncMode)event.getSyncMode());
	LOG.debug("---------------------- %s END (mode \"%s\") ----------------------", event.getSourceName(), syncMode);

    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SYNCSOURCE_END, 
                NULL, (LPARAM) syncSourceNameToIndex(event.getSourceName()));
}



void OutlookSyncSourceListener::syncSourceSyncModeRequested (SyncSourceEvent& event) {

    char* syncMode = syncModeName((SyncMode)event.getSyncMode());
    LOG.debug("Server requested syncmode \"%s\" for %s.", syncMode, event.getSourceName());
}


void OutlookSyncSourceListener::syncSourceTotalClientItems (SyncSourceEvent& event) {
    
    LOG.info("Total %s items to send: %d", event.getSourceName(), event.getData());

    SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SYNC_TOTALITEMS, 
                (WPARAM) 0, (LPARAM)event.getData());
}


void OutlookSyncSourceListener::syncSourceTotalServerItems (SyncSourceEvent& event) {
    
    if (event.getData() != -1) {
        LOG.info("Total %s items to receive: %d", event.getSourceName(), event.getData());
        SendMessage(HwndFunctions::getWindowHandle(), ID_MYMSG_SYNC_TOTALITEMS, 
                    (WPARAM) 1, (LPARAM)event.getData());
    }
}
