 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_DEF_WIN_CONFIG
#define INCL_DEF_WIN_CONFIG

/** @cond OLPLUGIN */
/** @addtogroup ClientDLL */
/** @{ */

#include "base/fscapi.h"
#include "base/Log.h"
#include "spds/AccessConfig.h"
#include "spds/DeviceConfig.h"
#include "spds/SyncSourceConfig.h"
#include "spds/SyncManagerConfig.h"
#include "spds/DefaultConfigFactory.h"

#include <string>

/**
 * This class is a factory of default configurations.
 * It can be inherited to define client specific Config parameters.
 */
class DefaultWinConfigFactory : public DefaultConfigFactory {

    public:

        DefaultWinConfigFactory();
        ~DefaultWinConfigFactory();

        /**
         * Returns a default generated AccessConfig for Win32.
         * @return  AccessConfig pointer allocated new, so it must
         *          be freed by the caller.
         */
        static AccessConfig*  getAccessConfig();

        /**
         * Returns a default generated DeviceConfig for Win32.
         * @return  DeviceConfig pointer allocated new, so it must
         *          be freed by the caller.
         */
        static DeviceConfig*  getDeviceConfig();

        /**
         * Returns a default generated SyncSourceConfig for Win32 (common props of SyncSourceConfig).
         * @param name : the name of syncsource desired
         * @return       WindowsSyncSourceConfig pointer allocated new, so it must
         *               be freed by the caller.
         */
        static SyncSourceConfig* getSyncSourceConfig(const std::wstring& name);

        /**
         * Returns a default generated WindowsSyncSourceConfig for Win32 (specific client props).
         * @param name : the name of syncsource desired
         * @param sc   : pointer to the SyncSourceConfig object where all common properties are stored (mandatory not NULL)
         * @return       WindowsSyncSourceConfig pointer allocated new, so it must
         *               be freed by the caller.
         */
        static WindowsSyncSourceConfig* getWinSyncSourceConfig(const std::wstring& name, SyncSourceConfig* sc);
};

/** @} */
/** @endcond */
#endif
