 /*
 * Copyright (C) 2006-2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY, TITLE, NONINFRINGEMENT or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
*/

#ifndef INCL_UTILS_DLL
#define INCL_UTILS_DLL

/** @cond OLPLUGIN */
/** @addtogroup utils */
/** @{ */

#include "base/fscapi.h"
#include "base/log.h"
#include "spds/SyncReport.h"
#include "WindowsSyncSource.h"
#include "outlook/ClientRecurrence.h"

#include <string>

/// Codes used by UI ("0" is the initialization)
#define SYNCSOURCE_CALENDAR     1
#define SYNCSOURCE_CONTACTS     2
#define SYNCSOURCE_NOTES        3
#define SYNCSOURCE_TASKS        4


/*-----------utils functions -----------------*/

// Open a message box with a timeout
int TimedMessageBox(HWND hwndOwner, LPCTSTR pszMessage, LPCTSTR pszTitle, UINT flags, DWORD dwTimeout);
// Display a message box only if not scheduled sync.
int safeMessageBox (const char*   message, const char*   title = NULL, unsigned int flags = 0);
int wsafeMessageBox(const WCHAR* wmessage, const WCHAR* wtitle = NULL, unsigned int flags = 0);

_declspec(dllexport) void printLog(const char* msg,  const char* level = LOG_INFO);
_declspec(dllexport) void printLog(const WCHAR* msg, const char* level = LOG_INFO);

bool  isSIF            (const std::wstring& dataType);
bool  isSIF            (const std::string&  dataType);
bool isAcceptedDataType(const std::wstring& dataType);
bool isAcceptedDataType(const std::string&  dataType);
char* syncModeName     (SyncMode code);
bool isFullSyncMode    (SyncMode mode);

void  toWindows        (char* str);
WCHAR* readAppDataPath ();
WCHAR* readDataPath    (const WCHAR* itemType);
int    makeDataDirs    ();
int    getWindowsUser  (std::wstring& userName);
int    getWindowsUserEx(std::wstring& userName);

std::wstring readFromFile          (const std::wstring& filePath);
int          writeToFile           (const std::wstring& content, const std::wstring& filePath, const WCHAR* mode = L"w");
int          writeToFile           (const std::string&  content, const std::string&  filePath, const char*  mode = "w");
std::string  getSyncMutexName      ();
char*        readSystemErrorMsg    (DWORD errorCode = 0);
std::wstring getSafeItemName       (ClientItem* cItem);
void         printReport           (SyncReport* sr, WindowsSyncSource** sources);
char*        friendlyName          (const char* sourceName);

int          getBuildNumberFromVersion(const char* swv);
long         variantTimeToTimeStamp   (const double vTime);

int syncSourceNameToIndex(const char*);



/**
 * Is the given status code an error status code? Error codes are the ones
 * outside the range 200-299.
 *
 * @param status the status code to check
 */
inline static bool isErrorStatus(int status) {
    return (status) && ((status < 200) || (status > 299));
}

/** @} */
/** @endcond */
#endif